@@ -1,16 +1,11 @@
+import json
 import asyncio
-import importlib
-import logging
-import functools
 
 from contextlib import contextmanager, asynccontextmanager
-from pathlib import Path
 
-import jinja2
+from roy.utils.tasks import TasksManager, register
 
-from roy.utils.tasks import Tasks, TasksManager, TaskRunError, register
-
-from .settings import SETTINGS, DeployComponentSettings
+from .settings import SETTINGS
 from .tasks import DeployTasks
 
 
@@ -23,7 +18,7 @@ class DeployTasksManager(TasksManager):
         self.hosts = {}
         self.nohost = {
             'user': '__nohost__',
-            'public_ip': None, 'private_ip': None, 'port': 0
+            'public_ip': None, 'private_ip': None, 'ssh_port': 22
         }
 
         for class_ in SETTINGS.tasks_classes:
@@ -35,10 +30,20 @@ class DeployTasksManager(TasksManager):
             commands.remove('-f')
             self.override = True
 
+        old_hosts_file, old_hosts = SETTINGS.settings_cache_file, {}
+        if self.override:
+            old_hosts_file.unlink(missing_ok=True)
+        elif old_hosts_file.exists():
+            old_hosts = json.loads(old_hosts_file.read_text())
+
         for provider_class in SETTINGS.provider_classes:
-            provider = provider_class(self, SETTINGS.services)
+            settings = SETTINGS.providers.get(provider_class.NAME, {})
+            provider = provider_class(
+                self, SETTINGS.hosts, old_hosts, settings)
             self.hosts.update(asyncio.run(provider.initialize()))
 
+        SETTINGS.settings_cache_file.write_text(
+            json.dumps(self.hosts, indent=2))
         super().run(*commands)
 
     def run_hooks(self, task_class, name, hook_name, instance=None):
@@ -71,8 +76,12 @@ class DeployTasksManager(TasksManager):
         if is_nohost:
             hosts = [self.nohost.copy()]
         else:
-            hosts = self.hosts[task_class.get_namespace()]
-        
+            hosts = [
+                host
+                for host in self.hosts.values()
+                if task_class.get_namespace() in host['components']
+            ]
+
         if is_firsthost:
             return hosts[:1]
 
